from redis import StrictRedis,ConnectionPool,Connection
import sys

sys.path.append('../../')
from inhand.service.SettingService import Redis
from inhand.dto.ErrorException import ErrorCodeException

class CacheService:
    def __init__(self,settings):
        self.settings = settings

    def connect(self):
        pass

    def close(self):
        pass

    def put(self,key,value,expire=None):
        raise (NameError, "Unsupported!")

    def get(self, key):
        raise (NameError, "Unsupported!")

    def exists(self, key):
        raise (NameError, "Unsupported!")

    @staticmethod
    def createCacheService(settings):
        if isinstance(settings,Redis):
            return RedisCacheService(settings)
        else:
            raise (NameError, "Unsupported cache type: {}".format(settings.database.type))

class RedisCacheService(CacheService):

    def __init__(self,settings):
        super().__init__(settings)
        self.cache=StrictRedis(host=settings.host,
                             port=settings.port,
                             db=settings.database,
                             password=settings.password)

    def put(self,key,value,expire=None):
        if expire is not None:
            self.cache.setex(key,expire,value)
        else:
            self.cache.set(key,value)

    def get(self, key):
        if self.exists(key):
            return  self.cache.get(key)
        else:
            raise ErrorCodeException(404,"no such key")

    def exists(self, key):
        return self.cache.exists(key)

class RedisCache(RedisCacheService):
    def __init__(self,cache:StrictRedis):
        self.cache=cache

    def put(self,key,value,expire=None):
        if expire is not None:
            self.cache.setex(key,expire,value)
        else:
            self.cache.set(key,value)

    def get(self, key):
        if self.exists(key):
            return  self.cache.get(key)
        else:
            raise ErrorCodeException(404,"no such key")

    def exists(self, key):
        return self.cache.exists(key)

class RedisPool:
    def __init__(self,settings,max_connections:int):
        self.pool=ConnectionPool(host=settings.host,
                             port=settings.port,
                             db=settings.database,
                             password=settings.password,socket_timeout=30, socket_connect_timeout=30,
                                socket_keepalive=True, encoding='utf-8',health_check_interval=60,
        max_connections=max_connections)

    def get_connection(self):
        return RedisCache(StrictRedis(connection_pool=self.pool))

    def close(self,connnection:StrictRedis):
        self.pool.disconnect(connnection)

"""if __name__ == '__main__':
    pool = RedisPool(host="redis",port=6379,db=2,password="Uxid7apUdks",max_connections=6)

    redis = pool.get_connection()
    redis.set("test","test")
    s = redis.get("test")

    print(s)"""



import json

from datetime import datetime
from bson import ObjectId

def bson_objectId_encoder(obj):
    if isinstance(obj, ObjectId):
        return str(obj)
    return obj

custom_encoder={
    ObjectId: bson_objectId_encoder,
}

class JSONEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime):
            return str(obj)
        elif isinstance(obj, ObjectId):
            return str(obj)
        else:
            return super(JSONEncoder, self).default(obj)


from fastapi import Request

from inhand.dto.ErrorException import APIError,Error, ErrorCodeException, ERR_CODE_400, ERR_CODE_403, ERR_CODE_404,ERR_CODE_500
from inhand.utilities.StringUtil import StringUtil

class HttpUtil:
    @staticmethod
    def parsePowerISApiHeader(request: Request) -> dict:
        base64str = request.headers.get("x-nezha-extra")
        if base64str is None:
            raise APIError(error=Error(code=ERR_CODE_403,message="Not a trust client, you need call the API with a apk key to the API gateway"))
        extra_dict=StringUtil.parseXAPIExtra(base64str)
        if 'app_id' not in extra_dict.keys():
            raise APIError(error=Error(code=ERR_CODE_403,message="Not a trust client"))
        return extra_dict



import json
from _datetime import datetime
from pydantic import BaseModel



class Metadata(BaseModel):
    created_at: datetime = None
    updated_at: datetime = None
    created_by: str = None
    updated_by: str = None


    def updateby(self, by: str):
        self.updated_by = by
        self.updated_at = datetime.now()

    def toDict(self):
        kv = {}
        if self.updated_by is not None:
            kv['updated_by'] = self.updated_by
            kv['updated_at'] = self.updated_at

        kv['created_by'] = self.created_by
        kv['created_at'] = self.created_at
        return kv

    @staticmethod
    def fromDict(kv: dict):
        metadata = Metadata(**kv)
        return metadata

    def tostring(self):
        json_string = json.dumps(self.toDict());
        return json_string

class Model(BaseModel):
    metadata: Metadata = None

    def updatedby(self, by: str):
        if self.metadata is None:
            self.metadata = Metadata({"reated_by":by, "updated_by":by})
        else:
            self.metadata.updated_by = by
            self.metadata.updated_at = datetime.now()